// $Id: kaddressbooktableview.cpp,v 1.10 2002/04/24 22:04:20 cschumac Exp $

#include <qlayout.h>
#include <qvbox.h>
#include <qlistbox.h>
#include <qwidget.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qcombobox.h>
#include <qtooltip.h>
#include <qdialog.h>
#include <qheader.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qdragobject.h>
#include <qevent.h>
#include <qurl.h>

#include <kurl.h>
#include <kdebug.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kcolorbutton.h>
#include <kglobal.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kurlrequester.h>
#include <klineedit.h>

#include "kaddressbooktableview.h"
#include "entryeditorwidget.h"
#include "viewoptions.h"
#include "contactentry.h"
#include "contactentrylist.h"
#include "browserentryeditor.h"
#include "undocmds.h"
#include "contactlistview.h"

KAddressBookTableView::KAddressBookTableView( ContactEntryList *cel,
		      QWidget *parent,
		      const char *name )
  : KAddressBookView(cel, parent, name), mDocument(cel)
{
  mainLayout = new QVBoxLayout( viewWidget(), 2 );
  
  // The list view will be created when the config is read.
  mListView = 0;
  
//  ldapSearchDialog = 0;
//  ldw = NULL;
}

KAddressBookTableView::~KAddressBookTableView()
{
}

/*
void KAddressBookTableView::selectNames( QStringList newFields )
{
    
  field.clear();
  fieldWidth.clear();
  if (newFields.count() == 0) {
    newFields += "X-FileAs";
    newFields += "EMAIL";
    newFields += "X-BusinessPhone";
    newFields += "X-HomePhone";
  }

  QStringList::Iterator it;
  for(it = newFields.begin(); it != newFields.end(); ++it)
  {
    field += *it;
    if (*it == "X-FileAs")
      fieldWidth += 180;
    else if (*it == "EMAIL")
      fieldWidth += 160;
    else
      fieldWidth += 120;
  }
}
*/

ContactEntryList *KAddressBookTableView::contactEntryList()
{
  return mDocument;
}

ContactListView *KAddressBookTableView::listView()
{
  return mListView;
}
/*
void KAddressBookTableView::showSelectNameDialog()
{
  SelectFields* sel = new SelectFields( field, this, "select", TRUE );
  if (!sel->exec())
    return;
  // Can't undo this action maybe should show a warning.
  //1  UndoStack::instance()->clear();
  //1  RedoStack::instance()->clear();

  selectNames( sel->chosenFields() );
  reconstructListView();
  delete sel;
}
*/
/*
void KAddressBookTableView::defaultSettings()
{
  selectNames( QStringList() );
  mListView->backPixmapOn = false;
  mListView->underline = true;
  mListView->autUnderline = true;
  mListView->tooltips_ = true;
  reconstructListView();
}
*/
void KAddressBookTableView::reconstructListView()
{
    if (mListView)
    {
        disconnect(mListView, SIGNAL(selectionChanged()),
                   this, SLOT(addresseeSelected()));
        disconnect(mListView, SIGNAL(executed(QListViewItem*)),
                   this, SLOT(addresseeExecuted(QListViewItem*)));
        
        delete mListView;
    }

  mListView = new ContactListView( this, viewWidget() );
  
  // Add the columns
  KABC::Field::List fieldList = fields();
  KABC::Field::List::ConstIterator it;
  for( it = fieldList.begin(); it != fieldList.end(); ++it ) {
      mListView->addColumn( (*it)->label() );
  }
 
  connect(mListView, SIGNAL(selectionChanged()),
          this, SLOT(addresseeSelected()));
  connect(mListView, SIGNAL(executed(QListViewItem*)),
          this, SLOT(addresseeExecuted(QListViewItem*)));
  refresh();
  
  mListView->setSorting( 0, true );
  mainLayout->addWidget( mListView );
  mainLayout->activate();
  mListView->show();
}

void KAddressBookTableView::change( QString entryKey, ContactEntry *ce )
{
  ContactListViewItem *plvi = mListView->getItem( entryKey );
  ContactEntry *oldce = mDocument->find( entryKey );
  if (plvi && oldce) {
    PwEditCommand *command = new PwEditCommand( mDocument, entryKey, oldce, ce );
    UndoStack::instance()->push( command );
    RedoStack::instance()->clear();
    
    refresh(entryKey);
  }
  else {
    PwNewCommand *command = new PwNewCommand( mDocument, ce );
    UndoStack::instance()->push( command );
    RedoStack::instance()->clear();
    
    refresh(entryKey);
  }
}

void KAddressBookTableView::writeConfig(KConfig *config)
{
  KAddressBookView::writeConfig(config);

  mListView->saveLayout(config, config->group());
}

void KAddressBookTableView::readConfig(KConfig *config)
{
  KAddressBookView::readConfig(config);
  
  // The config could have changed the fields, so we need to reconstruct
  // the listview.
  reconstructListView();
  
  // Set the alternating color if users want it
  mListView->setAlternateBackgroundEnabled(config->readBoolEntry("ABackground",
                                                                 true));
  mListView->setSingleLineEnabled(config->readBoolEntry("SingleLine", false));
  mListView->setToolTipsEnabled(config->readBoolEntry("ToolTips", true));
  
  if (config->readBoolEntry("Background", false))
    mListView->setBackgroundPixmap(config->readEntry("BackgroundName"));
  
  // Restore the layout of the listview
  mListView->restoreLayout(config, config->group());
}

/*
void KAddressBookTableView::viewOptions()
{
  QTabDialog* dlg = new QTabDialog( this );  
  ViewOptions *vo = new ViewOptions( mListView->backPixmapOn,
				     mListView->backPixmap,
				     mListView->underline,
				     mListView->autUnderline,
				     mListView->cUnderline,
				     mListView->tooltips(),
				     0,
				     "ViewOptions" );


  dlg->addTab( vo, i18n( "View Options" ) );
  
  dlg->setOkButton( i18n( "&OK" ) );
  dlg->setCancelButton( i18n( "Cancel" ) );

  // Show config dialog
  if (!dlg->exec())
    return;

  // Read information from config dialog and write to KConfig object
  mListView->backPixmapOn = vo->ckBackPixmap->isChecked();
  mListView->backPixmap = vo->pixmapPath->lineEdit()->text();
  mListView->underline = vo->ckUnderline->isChecked();
  mListView->autUnderline = vo->ckAutUnderline->isChecked();
  mListView->cUnderline = vo->kcbUnderline->color();
  mListView->tooltips_ = vo->ckTooltips->isChecked();
  mListView->loadBackground();
  mListView->saveConfig();
  mListView->triggerUpdate();

  config->sync();
  delete vo;
}
*/

void KAddressBookTableView::refresh(QString uid)
{
    // For now just repopulate. In reality this method should 
    // check the value of uid, and if valid iterate through
    // the listview to find the entry, then tell it to refresh.
  
    if (uid == QString::null)
    {
        // Clear the list view
        mListView->clear();
    
        // Add all the addressees. This does not currently use the filter, but
        // it should! -mpilone
        QStringList keys = addresseeUids();
        QStringList::Iterator it;
        for (it = keys.begin(); it != keys.end(); ++it )
        {
            new ContactListViewItem(*it,mListView,mDocument,fields());
        }
        
        emit selected(QString::null);
        
        // Sometimes the background pixmap gets messed up when we add lots
        // of items.
        mListView->repaint();
    }
    else
    {
        // Only need to update on entry. Iterate through and try to find
        // it
        QListViewItem *item;
        ContactListViewItem *ceItem;
        bool found = false;
        for (item = mListView->firstChild(); item && !found; 
             item = item->itemBelow()) {
            ceItem = dynamic_cast<ContactListViewItem*>(item);
            if ((ceItem != 0L) && (ceItem->entryKey() == uid)) {
                found = true;
                ceItem->refresh();
            }
        }
    }
}

QStringList KAddressBookTableView::selectedUids()
{
    QStringList uidList;
    QListViewItem *item;
    ContactListViewItem *ceItem;
    
    for(item = mListView->firstChild(); item; item = item->itemBelow()) 
    {
        if (mListView->isSelected( item ))
        {
            ceItem = dynamic_cast<ContactListViewItem*>(item);
            if (ceItem != 0L)
                uidList << ceItem->entryKey();
        }
    }
    
    return uidList;
}

void KAddressBookTableView::setSelected(QString uid, bool selected)
{
    QListViewItem *item;
    ContactListViewItem *ceItem;
    
    if (uid == QString::null)
    {
        mListView->selectAll(selected);
    }
    else
    {
        for(item = mListView->firstChild(); item; item = item->itemBelow())
        {
            ceItem = dynamic_cast<ContactListViewItem*>(item);
            if ((ceItem != 0L) && (ceItem->entryKey() == uid))
            {
                mListView->setSelected(item, selected);
                
                if (selected)
                    mListView->ensureItemVisible(item);
            }
        }
    }
}

void KAddressBookTableView::incrementalSearch(const QString &value, 
                                              KABC::Field *field)
{
    KABC::Field::List fieldList = fields();
    KABC::Field::List::ConstIterator it;
    int column = 0;
    for( it = fieldList.begin(); it != fieldList.end(); ++it ) {
      if ( (*it)->equals( field ) ) break;
      ++column;
    }
    
    if ( it == fieldList.end() ) column = 0;

    // Now do the inc search
    QListViewItem *item = mListView->findItem(value, column, Qt::BeginsWith);
    
    if (item)
    {
        // We have a match. Deselect all the others and select this one
        mListView->clearSelection();
        mListView->setSelected(item, true);
        mListView->ensureItemVisible(item);
    }
}

void KAddressBookTableView::addresseeSelected()
{
    // We need to try to find the first selected item. This might not be the
    // last selected item, but when QListView is in multiselection mode,
    // there is no way to figure out which one was
    // selected last.
    QListViewItem *item;
    bool found =false;
    for (item = mListView->firstChild(); item && !found; 
         item = item->nextSibling())
    {
        if (item->isSelected())
        {
            found = true;
            ContactListViewItem *ceItem 
                 = dynamic_cast<ContactListViewItem*>(item);
             emit selected(ceItem->entryKey());
        }
    }
    
    if (!found)
        emit selected(QString::null);
}

void KAddressBookTableView::addresseeExecuted(QListViewItem *item)
{
    if (item)
    {
        ContactListViewItem *ceItem 
               = dynamic_cast<ContactListViewItem*>(item);
               
       if (ceItem)
       {
           emit executed(ceItem->entryKey());
       }
    }
    else
    {
        emit executed(QString::null);
    }
}

#include "kaddressbooktableview.moc"
